import platform
import shutil
import psutil
import socket

from .cuda_check import check_cuda
from .network_check import check_internet, check_proxy


class Checker:
    """
    Core System Checker
    Old features preserved, new features extended
    """

    def basic_system_info(self):
        return {
            "os": platform.system(),
            "os_version": platform.version(),
            "hostname": socket.gethostname(),
            "architecture": platform.machine(),
            "processor": platform.processor(),
            "python_version": platform.python_version(),
        }

    def cpu_info(self):
        return {
            "physical_cores": psutil.cpu_count(logical=False),
            "total_cores": psutil.cpu_count(logical=True),
            "cpu_usage_percent": psutil.cpu_percent(interval=1)
        }

    def memory_info(self):
        memory = psutil.virtual_memory()
        return {
            "total_gb": round(memory.total / (1024 ** 3), 2),
            "available_gb": round(memory.available / (1024 ** 3), 2),
            "used_percent": memory.percent
        }

    def disk_info(self):
        total, used, free = shutil.disk_usage("/")
        return {
            "total_gb": round(total / (1024 ** 3), 2),
            "used_gb": round(used / (1024 ** 3), 2),
            "free_gb": round(free / (1024 ** 3), 2),
            "used_percent": round((used / total) * 100, 2)
        }

    def cuda_performance(self):
        return check_cuda()

    def internet_status(self):
        return check_internet()

    def proxy_status(self):
        return check_proxy()

    def full_report(self):
        return {
            "system": self.basic_system_info(),
            "cpu": self.cpu_info(),
            "memory": self.memory_info(),
            "disk": self.disk_info(),
            "cuda": self.cuda_performance(),
            "internet": self.internet_status(),
            "proxy": self.proxy_status()
        }



from envhealth import Checker, Reporter


def test_reporter_outputs():
    chk = Checker()
    data = chk.full_report()

    rep = Reporter(data)

    assert isinstance(rep.pretty_text(), str)
    assert isinstance(rep.to_json(), str)
    assert "<html>" in rep.to_html()



import json
from .pdf_report import export_pdf


class Reporter:
    def __init__(self, data):
        self.data = data

    def to_json(self):
        return json.dumps(self.data, indent=4)

    def pretty_text(self):
        out = []
        for section, values in self.data.items():
            out.append(f"\n=== {section.upper()} ===")
            for k, v in values.items():
                out.append(f"{k}: {v}")
        return "\n".join(out)

    def to_html(self):
        html = ["<html><body><h1>EnvHealth Report</h1>"]
        for section, values in self.data.items():
            html.append(f"<h2>{section.upper()}</h2><ul>")
            for k, v in values.items():
                html.append(f"<li><b>{k}</b>: {v}</li>")
            html.append("</ul>")
        html.append("</body></html>")
        return "\n".join(html)

    def to_pdf(self, filename="envhealth_report.pdf"):
        export_pdf(self.data, filename)
        return filename



def check_cuda():
    try:
        import torch
        cuda_available = torch.cuda.is_available()

        result = {
            "cuda_available": cuda_available
        }

        if cuda_available:
            device = torch.device("cuda")
            import time

            x = torch.randn((8000, 8000), device=device)
            start = time.time()
            y = x @ x
            torch.cuda.synchronize()
            end = time.time()

            result["gpu_name"] = torch.cuda.get_device_name(0)
            result["benchmark_time_sec"] = round(end - start, 4)

        return result

    except Exception as e:
        return {"cuda_available": False, "error": str(e)}



import argparse
from .checker import Checker
from .reporter import Reporter


def main():
    parser = argparse.ArgumentParser(description="EnvHealth System Diagnostics")

    parser.add_argument("--json", action="store_true", help="Output report in JSON")
    parser.add_argument("--html", action="store_true", help="Output report in HTML format")
    parser.add_argument("--pdf", action="store_true", help="Export report as PDF")

    args = parser.parse_args()

    chk = Checker()
    data = chk.full_report()
    rep = Reporter(data)

    if args.json:
        print(rep.to_json())
        return

    if args.html:
        html = rep.to_html()
        fname = "envhealth_report.html"
        with open(fname, "w") as f:
            f.write(html)
        print(f"HTML report generated: {fname}")
        return

    if args.pdf:
        fname = rep.to_pdf()
        print(f"PDF report generated: {fname}")
        return

    print(rep.pretty_text())



from envhealth import Checker


def test_full_report_structure():
    chk = Checker()
    report = chk.full_report()

    assert "system" in report
    assert "cpu" in report
    assert "memory" in report
    assert "disk" in report
    assert "cuda" in report
    assert "internet" in report
    assert "proxy" in report



import requests
import os


def check_internet():
    try:
        response = requests.get("https://www.google.com", timeout=3)
        return {
            "connected": True,
            "status_code": response.status_code
        }
    except Exception as e:
        return {"connected": False, "error": str(e)}


def check_proxy():
    proxies = {
        "http": os.getenv("HTTP_PROXY"),
        "https": os.getenv("HTTPS_PROXY")
    }

    return {
        "proxy_enabled": any(proxies.values()),
        "proxies": proxies
    }



from reportlab.lib.pagesizes import A4
from reportlab.pdfgen import canvas


def export_pdf(data, filename):
    c = canvas.Canvas(filename, pagesize=A4)
    width, height = A4

    y = height - 50
    c.setFont("Helvetica-Bold", 16)
    c.drawString(50, y, "Environment Health Report")
    y -= 40

    c.setFont("Helvetica", 12)

    for section, values in data.items():
        c.drawString(50, y, f"[{section.upper()}]")
        y -= 20

        for k, v in values.items():
            c.drawString(60, y, f"{k}: {v}")
            y -= 20

            if y < 80:
                c.showPage()
                y = height - 50

    c.save()
